#!/usr/bin/env python
# -*- coding: utf-8 -*-
########################################################################
#
# wpcmd.py
# 
# wpcmd - A WordPress command line client
# Copyright (c) 2009-2012 Elias Schwerdtfeger, http://www.tamagothi.de/ 
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
#
########################################################################

"""\
wpcmd.py

The command line driven WordPress client.

Copyright (c) 2009-2012 Elias Schwerdtfeger, http://www.tamagothi.de/

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
"""

from __future__ import print_function

# Important to do this before any other things are done...
import locale
locale.setlocale(locale.LC_ALL, '')

import sys
import optparse
import wpsh.commands as cmd
import wpsh.wpshbase



ID = '$Id: wpcmd.py,v 1.13 2012/04/27 20:28:25 elias Exp $'



def main():
    "Main function."
    p = optparse.OptionParser(usage='wpcmd [options] filelist',
                              version='wpcmd ' + wpsh.wpshbase.VERSION)
    p.add_option('-v', '--verbose',
                 action='store_true',
                 dest='verbose',
                 help='print messages while processing',
                 default=False)
    p.add_option('-f', '--verbose-file',
                 dest='verbose_file',
                 help='print verbose messages to a file while processing',
                 metavar='FILENAME',
                 default=False)
    p.add_option('-l', '--logfile',
                 dest='log_file',
                 help='File for log messages',
                 metavar='FILENAME',
                 default=False)
    # The legal stuff and other informations
    p.add_option('-C', '--copyleft',
                 action='store_true',
                 dest='copyleft',
                 help='read the license information for the program',
                 default=False)
    p.add_option('-W', '--warranty',
                 action='store_true',
                 dest='warranty',
                 help='read about the (no-)warranty for the program',
                 default=False)
    p.add_option('-S', '--support',
                 action='store_true',
                 dest='support',
                 help='read how to support me',
                 default=False)
    # Read documentation
    p.add_option('-D', '--documentation',
                 action='store_true',
                 dest='documentation',
                 help='read the documentation in your browser',
                 default=False)
    # Configuration options
    p.add_option('-L', '--bloglist',
                 action='store_true',
                 dest='listblogs',
                 help='show the names of the configured blogs', 
                 default=False)
    p.add_option('-A', '--blogadd', 
                 action='store_true',
                 dest='addblog',
                 help='validate a blog and add it to the configuration',
                 default=False)
    p.add_option('-R', '--blogremove',
                 metavar='BLOGNAME',
                 action='store',
                 dest='removeblog',
                 help='remove a blog from the configuration',
                 default=False)
    p.add_option('-I', '--bloginfo',
                 metavar='BLOGNAME',
                 action='store',
                 dest='bloginfo',
                 help='show some blog informations',
                 default=False)
    p.add_option('-G', '--blogrevalidate',
                 metavar='BLOGNAME',
                 action='store',
                 dest='revalidateblog',
                 help='revalidate an already configured blog',
                 default=False)
    p.add_option('-K', '--blogcategorytree',
                 metavar='BLOGNAME',
                 action='store',
                 dest='blogcategorytree',
                 help='print the category tree for the blog',
                 default=False)
    p.add_option('-T', '--blogposttemplate',
                 metavar='BLOGPATTERN',
                 action='store',
                 dest='blogposttemplate',
                 help='create post templates for blogs matching pattern',
                 default=False)
    p.add_option('-P', '--blogpassword',
                 metavar='BLOGNAME',
                 action='store',
                 dest='blogpassword',
                 help='change the password for a blog',
                 default=False)
    # Other funny stuff
    p.add_option('-V', '--blogvisit',
                 metavar='BLOGNAME',
                 action='store',
                 dest='visitblog',
                 help='visit the blog with your webbrowser',
                 default=False)
    p.add_option('-X', '--configcrypt',
                 metavar='STRING',
                 action='store',
                 dest='configcryptstring',
                 help='Crypt string for configuration',
                 default=False)
    try:
        (opt, a) = p.parse_args()
    except optparse.OptParseError:
        p.print_help()
        exit('Illegal option')
    # General options
    cmd.verbose = opt.verbose
    if opt.verbose_file:
        cmd.setVerboseFile(opt.verbose_file.decode(wpsh.wpshbase.LOCALENC))
    if opt.log_file:
        wpsh.wpshbase.changeLogPath(opt.log_file)
    if opt.copyleft:
        print(wpsh.wpshbase.LICENSE)
        sys.exit()
    if opt.warranty:
        print(wpsh.wpshbase.WARRANTY)
        sys.exit()
    if opt.support:
        print(wpsh.wpshbase.DONATE)
        sys.exit()
    if opt.configcryptstring:
        cmd.cryptConfigString(opt.configcryptstring)
        sys.exit()
    # Read user-orientented documentation
    if opt.documentation:
        cmd.visitTutorial()
        sys.exit()
    # Blog configuration options
    if opt.listblogs:
        cmd.listBlogs()
    elif opt.addblog:
        # The parameters may be given at command line, especally from
        # lazy people like me...
        while len(a) < 5:
            a.append(None)
        cmd.addBlog(a[0], a[1], a[2], a[3], a[4])
        sys.exit()
    elif opt.removeblog:
        cmd.removeBlog(opt.removeblog)
        sys.exit()
    elif opt.visitblog:
        cmd.visitBlog(opt.visitblog)
        sys.exit()
    elif opt.bloginfo:
        cmd.blogInformation(opt.bloginfo)
        sys.exit()
    elif opt.revalidateblog:
        cmd.revalidateBlog(opt.revalidateblog)
        sys.exit()
    elif opt.blogcategorytree:
        cmd.categoryTree(opt.blogcategorytree)
        sys.exit()
    elif opt.blogposttemplate:
        cmd.blogPostTemplate(opt.blogposttemplate)
        sys.exit()
    elif opt.blogpassword:
        cmd.changePassword(opt.blogpassword)
        sys.exit()
    else:
        if len(a) > 0:
            cmd.enforceReturnValueOnException()
            for fname in a:
                cmd.postFile(fname.decode(wpsh.wpshbase.LOCALENC))
        else:
            cmd.interactiveMode()



if __name__ == '__main__':
    main()



########################################################################
#
# $Log: wpcmd.py,v $
# Revision 1.13  2012/04/27 20:28:25  elias
# Cleanup for 0.5 release
#
# Revision 1.12  2011/02/23 09:06:16  elias
# Print-Funktion verwendet...
#
# Revision 1.11  2010/09/29 20:11:44  elias
# Added -l option to set a logfile
#
# Revision 1.10  2010/09/26 19:07:44  elias
# Added the new -d option for reading HTML-documentation.
#
# Revision 1.9  2010/09/26 16:39:24  elias
# Option for verbose message file
# (Output redirection in the shell with unicode raises errors)
#
# Revision 1.8  2010/09/24 22:27:18  elias
# Source cleanup for beta release
#
# Revision 1.7  2010/09/24 17:19:52  elias
# Added -P option to change the password used by wpcmd
#
# Revision 1.6  2010/02/10 19:02:54  elias
# Added -X option.
#
# Revision 1.5  2009/09/23 16:04:42  elias
# Source cleanup for Alpha2.
#
# Revision 1.4  2009/09/20 20:18:17  elias
# Source cleanup for alpha release.
#
# Revision 1.3  2009/09/20 02:32:20  elias
# Post functionality.
# Upload functionality.
# Possibility to blog from stdin.
#
# Revision 1.2  2009/09/18 17:01:31  elias
# Intermediate state.
#
# Revision 1.1  2009/09/18 13:34:10  elias
# Initial revision
#
########################################################################
#
# CODE IS PROSA, NOT POETRY!
#
########################################################################
