#!/usr/bin/env python
# -*- coding: utf-8 -*-
########################################################################
#
# wpsh/blog.py
#
# wpcmd - A WordPress command line client
# Copyright (c) 2009-2010 Elias Schwerdtfeger, http://www.tamagothi.de/ 
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
#
########################################################################

"""\
blog.py

The interface for blog configuration.
"""


import re
import os
import os.path
import webbrowser
import urllib
import codecs
import cPickle
import xml.dom.minidom
import mimetypes

import wpshbase
import wptypes



ID='$Id: blog.py,v 1.16 2012/04/27 02:49:29 elias Exp $'



PASSWORD_ENCODING = 'rot13'
PasswordCodec = codecs.lookup(PASSWORD_ENCODING)
UPLOAD_TRIES = 3



class NoSuchBlog(wpshbase.Error):
    """Blog doesn't exist or isn't configured"""
    pass



class InvalidConfiguration(wpshbase.Error):
    """Blog isn't valid"""
    pass



class BlogError(wpshbase.Error):
    """General blog communication error"""
    pass



class NoRSD(BlogError):
    """Failed to read RSD-URL from the blog's HTML"""
    pass



class ValidateError(BlogError):
    """Failed to validate the settings"""
    pass



class XMLRPCError(BlogError):
    """General error in XMLRPC"""
    pass



class InputError(wpshbase.Error):
    """General input error"""
    pass



class DoubleTitle(InputError):
    """Title specified twice"""
    pass



class MissingTitle(InputError):
    """No title given"""
    pass



class DoubleType(InputError):
    """Type specified twice"""
    pass



class IllegalType(InputError):
    """Illegal type specification"""
    pass



class NoBlogContext(InputError):
    """Missing blog context"""
    pass



class MissingMeta(InputError):
    """Missing meta information"""
    pass



class ParserError(InputError):
    """An error in the input parser"""
    pass



class UnclosedPreTag(ParserError):
    """A <pre> tag wasn't closed"""
    pass
    


class MultiplePreTag(ParserError):
    """A <pre> tag contains a <pre> tag"""
    pass



class FileUploadFailed(XMLRPCError):
    """A file upload failed"""
    pass



class PublicationFailed(XMLRPCError):
    """Content publication failed"""
    pass



class BlogConfig(wpshbase.Base):
    """\
    Configuration for a blog.

    INSTANCE PROPERTIES:

    id - The blog ID
    name - The internal name of the blog
    url - The URL of the blog
    user - The username for logins
    pwd - The password for logins
    xmlurl - The XMLRPC entry URL
    cats - The list of configured categories
    admin - Is the user an administrator
    comment - A internal comment

    If you want to add a new blog, create an instance with the default
    constructor and fill the fields. Save it with the saveBlogConfig
    method of a Config instance.
    """

    def __init__(self, blogname=None, cfgobj=None):
        """\
        Constructor.

        Normally, the arguments are given by the framework. If you want
        to add a new blog, set the components manually and save it in 
        a Config file.

        The configureBlog() function is the easy way, used in wpcmd.

        NOTICE:

        The login informations are stored as NEARLY CLEAR TEXT in the
        configuration file, they are just rot13-encoded. The file is 
        readable for its user only, but it is a security issue, and 
        a WEAKNESS BY DESIGN. So don't use wpcmd on a system which
        is open to many people, if you want the password secret. (And
        you want!)
        """
        self.clear()
        self.logDebug('Constructor')
        if blogname and cfgobj:
            try:
                self.logDebug('Reading config for %s', blogname)
                self.name = blogname
                self.url = cfgobj.get(blogname, 'BlogURL')
                self.id = cfgobj.getint(blogname, 'BlogID')
                u = cfgobj.get(blogname, 'Username')
                p = cfgobj.get(blogname, 'Password')
                self.user, self.pwd = self.decryptLogin([u, p])
                self.xmlurl = cfgobj.get(blogname, 'XMLRPCURL')
                c = cfgobj.get(blogname, 'Categories')
                self.cats = cPickle.loads(c)
                self.admin = cfgobj.getboolean(blogname, 'CanAdmin')
                self.comment = cfgobj.get(blogname, 'Comment')
                self.__ok = True
            except:
                self.logWarning('No config section for %s', blogname)
                self.__ok = False

    def clear(self):
        """\
        Clear the configuration.
        """
        self.logDebug('Clearing blog configuration')
        self.__ok = False
        self.__xmlproxy = None
        self.__xmlproxyok = False
        self.__cattree = None
        self.name = ''
        self.url = ''
        self.id = 0
        self.user = ''
        self.pwd = ''
        self.xmlurl = ''
        self.cats = {}
        self.admin = False
        self.comment = ''

    def encryptLogin(self, lst):
        """\
        Encrypt the login information for storage.
        """
        return [PasswordCodec.encode(i)[0] for i in lst]
        
    def decryptLogin(self, lst):
        """\
        Decrypt the login information for storage.
        """
        return [PasswordCodec.decode(i)[0] for i in lst]

    def writeToConfigFile(self, cfgobj):
        """\
        Write the configuration to a config file object.

        This is a more internal function, used by various methods of
        the Config class.
        """
        self.logDebug('Writing configuration for %s', self.name)
        if not cfgobj.has_section(self.name):
            cfgobj.add_section(self.name)
        u, p = self.encryptLogin([self.user, self.pwd])
        c = cPickle.dumps(self.cats)
        cfgobj.set(self.name, 'BlogURL', self.url)
        cfgobj.set(self.name, 'BlogID', self.id)
        cfgobj.set(self.name, 'Username', u)
        cfgobj.set(self.name, 'Password', p)
        cfgobj.set(self.name, 'XMLRPCURL', self.xmlurl)
        cfgobj.set(self.name, 'Categories', c)
        cfgobj.set(self.name, 'CanAdmin', self.admin)
        cfgobj.set(self.name, 'Comment', self.comment)

    def isValid(self):
        """\
        Is this blog valid and configured?
        """
        if not self.__ok:
            return False
        elif not self.xmlurl:
            return False
        elif not self.user:
            return False
        elif not self.pwd:
            return False
        else:
            return True

    def getServerProxy(self):
        """\
        Get a XMLRPC server proxy.

        Returns the server proxy
        """
        # I cannot use a simple test to self.__xmlproxy and have to
        # keep an additional instance variable. Sometimes the python magic
        # is rather black magic...
        if not self.__xmlproxyok:
            self.logDebug('Creating XML-RPC server proxy for %s', self.name)
            if not self.isValid():
                raise InvalidConfiguration('BlogConfig is invalid')
            self.__xmlproxy = wpshbase.XmlRpc(self.xmlurl)
            self.__xmlproxyok = True
        return self.__xmlproxy

    def readCategories(self):
        """\
        Read the list of categories from the blog.

        On failure, the configured blog is marked as invalid.
        """
        self.logDebug('Reading the categories for %s', self.name)
        self.cats = {}
        x = self.getServerProxy()
        for c in x.metaWeblog.getCategories(self.id, self.user, self.pwd):
            cobj = wptypes.Category(c)
            self.cats[cobj.categoryName] = cobj
        self.__cattree = None

    def revalidate(self):
        """\
        Validate the blog.

        This is done internally by checking user access to the blog and
        reading the list of categories. If this succeeds, the blog config
        is valid.

        NOTICE:

        It will only work with new WordPress versions and uses some fresh
        stuff from the API. 

        Returns True on Success
        """
        self.logInfo('Revalidating %s', self.name)
        self.__ok = True
        x = self.getServerProxy()
        flag = False
        for b in x.wp.getUsersBlogs(self.user, self.pwd):
            if b['url'].startswith(self.url):
                self.admin = bool(b['isAdmin'])
                self.url = b['url']
                self.id = int(b['blogid'])
                flag = True
                break
        if flag:
            try:
                self.readCategories()
            except wpshbase.XmlFault:
                self.logException('Failed to read categories')
                self.__ok = False
        else:
            self.logError('Failed to validate %s', self.name)
            self.__ok = False
        return self.__ok

    def visit(self):
        """\
        Visit the blog with your web browser.
        """
        self.logDebug('Visiting %s with the browser', self.name)
        if not self.isValid():
            raise InvalidConfiguration('BlogConfig is invalid')
        webbrowser.open(self.url)

    def getCategoryTree(self):
        """\
        Get the category tree for this blog.

        Returns a CategoryTree instance.
        """
        if not self.isValid():
            raise InvalidConfiguration('BlogConfig is invalid')
        if not self.__cattree:
            self.logDebug('Creating category tree for %s', self.name)
            self.__cattree = wptypes.CategoryTree(self.cats)
        return self.__cattree

    def newCategory(self, cat):
        """\
        Create a new category.

        cat - A Category instance
        """
        self.logInfo('Creating new category %s for %s', 
                     cat.categoryName, self.name)
        if not self.isValid():
            raise InvalidConfiguration('BlogConfig is invalid')
        hsh = { 'name': cat.categoryName, 
                'description': cat.description }
        if cat.parentId:
            hsh['parent_id'] = cat.parentId
        x = self.getServerProxy()
        cat_id = x.wpshbase.newCategory(self.id, self.user, self.pwd, hsh)
        if not cat_id:
            self.logError('No category id returned from XML-RPC')
            raise XMLRPCError('strange internal error')
        cat.categoryId = cat_id
        self.cats[cat.categoryName] = cat
        self.__cattree = None

    def removeCategory(self, cat):
        """\
        Remove a category.

        cat - A Category instance
        """
        self.logDebug('Removing category %s from %s',
                      cat.categoryName, self.name)
        if not self.isValid():
            raise InvalidConfiguration('BlogConfig is invalid')
        x = self.getServerProxy()
        x.wp.deleteCategory(self.id, self.user, self.pwd, cat.categoryId)
        del self.cats[cat.categoryName]
        self.__cattree = None

    def __templateHelper(self, a):
        """\
        A helper method for creating templates.
        """
        a.insert(0, 'blog: %s' % (self.name,))
        a = ['!!' + i for i in a]
        a.insert(0, '# Template for %s' % (self.name,))
        a.insert(0, '# -*- coding: %s -*-' % (wpshbase.LOCALENC.lower(),))
        a.append('!!title: ')
        return '\n'.join(a) + '\n\n'

    def createPostTemplate(self):
        """\
        Create a post template for an instance.

        Returns string with the template
        """
        self.logDebug('Creating a post template for %s', self.name)
        a = ['type: post']
        # Do some funny formatting with the categories to make the
        # lines fit in 74 column displays and so easier to edit...
        offset = len('!!categories: ')
        catlist = self.cats.keys()
        catlist.sort()
        worker = []
        colcnt = 0
        for catname in catlist:
            catlen = len(catname) + 2
            if colcnt + catlen + offset > 74:
                a.append('categories: %s' % (', '.join(worker),))
                worker = [catname]
                colcnt = catlen
            else:
                worker.append(catname)
                colcnt += catlen
        if len(worker) > 0:
            a.append('categories: %s' % (', '.join(worker),))
        a.append('tags: ')
        return self.__templateHelper(a)

    def createPageTemplate(self):
        """\
        Create a page template for an instance.
        
        Returns string with the template
        """
        self.logWarning('Attempt to create a page template for %s', self.name)
        a = ['type: page']
        return self.__templateHelper(a)

    def __doUpload(self, filename, msgcb):
        """Internal: Publish an upload as part of a post."""
        self.logInfo('Uploading file %s to %s', filename, self.name)
        x = self.getServerProxy()
        if msgcb:
            msgcb('Uploading "%s"...', filename);
        s = { }
        fp = open(filename, 'rb')
        s['name'] = os.path.basename(filename)
        s['type'] = mimetypes.guess_type(filename)[0]
        s['bits'] = wpshbase.XmlBinary(fp.read())
        fp.close()
        triesleft = UPLOAD_TRIES
        while triesleft > 0:
            try:
                h = x.wp.uploadFile(self.id, self.user, self.pwd, s)
                triesleft = 0
            except:
                self.logException('Upload exception')
                triesleft -= 1
                if triesleft > 0:
                    if msgcb:
                        msgcb('Upload failed, retrying...')
                else:
                    self.logError('File upload failed')
                    raise FileUploadFailed(filename)
        self.logInfo('Upload successful, %s is %s on the server', 
                     filename, h['url'])
        return h['url']

    def publishContent(self, instance, msgcb=None):
        """Publish a content instance"""
        self.logInfo('Publishing content to %s', self.name)
        uploads = instance.getUploadArray()
        if len(uploads) > 0:
            for filename in uploads:
                upload_url = self.__doUpload(filename, msgcb)
                instance.registerUploadedPath(filename, upload_url)
            instance.replaceUploadUris()
        if msgcb:
            msgcb('Publishing text...')
        s = instance.getXMLRPCHash()
        x = self.getServerProxy()
        triesleft = UPLOAD_TRIES
        while triesleft > 0:
            try:
                post_id = x.metaWeblog.newPost(self.id, self.user, self.pwd, s)
                triesleft = 0
            except:
                self.logException('New post creation failed')
                triesleft -= 1
                if triesleft > 0:
                    if msgcb:
                        msgcb('Publication failed, retrying...')
                else:
                    self.logError('Publication failed')
                    raise PublicationFailed('Could not publish text')
        return int(post_id)



class Config(wpshbase.Base):
    """\
    Interface to the configuration of all blogs.

    In normal applications, there is only one instance of this class.
    The blogs are configured and accessed here.
    """

    def __init__(self):
        """\
        Constructor.
        """
        self.__c = wpshbase.CfgParser()
        self.__b = {}
        self.__cfgpath = wpshbase.CfgPath
        if os.path.exists(self.__cfgpath):
            self.__c.read(self.__cfgpath)
            for sect in self.__c.sections():
                self.__b[sect] = None

    def __save(self):
        """
        Save a new configuration to the config file.
        """
        self.logDebug('Writing to config file')
        oldumask = -1
        try:
            oldumask = os.umask(0177)
        except:
            self.logWarning('Couldn\'t change umask for config file')
            pass
        fp = open(self.__cfgpath, 'wb')
        self.__c.write(fp)
        fp.close()
        if oldumask != -1:
            os.umask(oldumask)

    def getBlogList(self):
        """\
        Get a list of configured blogs.

        Returns the configured blogs as a list.

        PLUGIN SLOTS:
        Config.GetBlogList - to filter the blog list
        """
        self.logDebug('Getting list of configured blogs')
        return self.__b.keys()
                
    def getBlogConfig(self, name):
        """\
        Get the blog configuration for a specified name.

        Returns the Blog object.
        """
        self.logDebug('Getting configuration for %s', name)
        if not self.__b.has_key(name):
            raise NoSuchBlog('No configuration for %s' % (name,))
        if not self.__b[name]:
            self.__b[name] = BlogConfig(name, self.__c)
        return self.__b[name]

    def saveBlogConfig(self, bc):
        """\
        Add a new or changed blog configuration.

        You have to fill the components manually.

        bc - The blog config object
        """
        self.logDebug('Saving configuration for %s', bc.name)
        if not bc.isValid():
            raise InvalidConfiguration('BlogConfig is invalid')
        self.__b[bc.name] = bc
        bc.writeToConfigFile(self.__c)
        self.__save()

    def renameBlogConfig(self, oldname, newname):
        """\
        Rename a blog configuration.

        oldname - the old name
        newname - the new name
        """
        self.logInfo('Renaming configuration %s to %s', oldname, newname)
        bc = self.getBlogConfig(oldname)
        self.__c.remove_section(oldname)
        del self.__b[oldname]
        bc.name = newname
        self.__b[newname] = bc
        self.saveBlogConfig(bc)

    def deleteBlogConfig(self, name):
        """\
        Delete a blog configuration.

        name - The name of the blog
        """
        self.logInfo('Deleting configuration for %s', name)
        if not self.__b.has_key(name):
            raise NoSuchBlog('%s is not configured' % (name,))
        del self.__b[name]
        self.__c.remove_section(name)
        self.__save()



theConfig = Config()



class RSDGetter(wpshbase.Base, wpshbase.HtmlParser):
    """\
    Get the RSD informations (EditURI) from the HTML source.

    A helper class.

    This works with a WordPress Blog, and isn't tested with any other
    system, but my script is written for WordPress, so don't expect it
    to work with other blog software.

    KNOWNBUG
    Does not work with non-ASCII characters in an URL currently.
    """

    def handle_starttag(self, tag, attr):
        """\
        Extract a link-tag with type application/rsd+xml.
        """
        if tag != 'link':
            return
        ah = {}
        for key, val in attr:
            ah[key] = val
        if not ah.has_key('type'):
            return
        if ah['type'].lower() == 'application/rsd+xml':
            if self.__found:
                raise RuntimeError('Double RSD spec')
            self.__found = ah['href']

    def getRSDFromURL(self, url):
        self.__found = None
        self.logInfo('Get RSD address from %s', url)
        self.reset()
        # KNOWN BUG!
        #
        # Must use codec to ignore errors with charsets. Oh, how I hate
        # this sucking python! Thanks Bog for the strict ASCII in
        # most URLs, but this will be problematic with punycode URLs
        # containing funny letters, so I have to fix it later. TODO.
        #
        # (An almost always working way of fixing is to assume that the 
        # charset is UTF-8, which is a WordPress default changed seldomly.
        # But I'm still thinking on better ways, and my alpha release can
        # be buggy.)
        f = codecs.EncodedFile(urllib.urlopen(url), 'ascii', 'ascii', 'ignore')
        self.feed(f.read())
        f.close()
        if not self.__found:
            self.logError('No RSD address found in %s', url)
        return self.__found



def configureBlog(name, url, user, pwrd, comment=''):
    """\
    Create a new BlogConfig object for a blog.
    """
    bc = BlogConfig()
    # Enforce http:// specification if not given.
    # Many folks do believe www. is a protocol... ;-)
    # But if someone wants https:, he must type it manually.
    if not url.startswith('http'):
        url = 'http://' + url
    # Get the XMLRPC entry URL
    # This is the part not working with older versions.
    rsdget = RSDGetter()
    rsdurl = rsdget.getRSDFromURL(url)
    if not rsdurl:
        raise NoRSD('Failed to read RSD from %s' % (url))
    # Parse the relevant stuff from the RSD DOMtree 
    # (very quick and dirty)
    f = urllib.urlopen(rsdurl)
    doc = xml.dom.minidom.parse(f)
    f.close()
    blogid = None
    for a in doc.getElementsByTagName('api'):
        if a.getAttribute('name') == 'WordPress':
            blogid = int(a.getAttribute('blogID'))
            xmlrpc = a.getAttribute('apiLink')
            break
    if not blogid:
        raise NoRSD('Is this a WordPress blog?')
    # Write it to the clean configuration
    bc.name = name
    bc.url = url
    bc.user = user
    bc.pwd = pwrd
    bc.id = blogid
    bc.xmlurl = xmlrpc
    bc.comment = comment
    # Validate this configuration
    if not bc.revalidate():
        raise ValidateError('Could not validate the settings')
    # And return it. That's simple, isn't it...
    return bc



class InputMetaPart(wpshbase.Base):
    """\
    A meta part of an input file.

    Every blog has a meta part, which is an representation of all 
    those !! lines in a file.

    COMPONENTS:
    meta - The raw meta data
    """

    def __init__(self, blogname):
        """\
        Constructor.

        blogname - The name of the blog
        """
        self.__blogname = blogname.strip()
        self.__meta = {}
        self.__bc = theConfig.getBlogConfig(blogname)

    def addMeta(self, metakey, metadata):
        """\
        Add a line of meta information to the part.

        metakey - The keyword
        metadata - The associated data
        """
        metadata = metadata.strip()
        # Do not add empty strings
        if len(metadata) == 0:
            return
        self.logDebug('Adding meta data %s=%s', metakey, metadata)
        metakey = metakey.lower()
        if not self.__meta.has_key(metakey):
            self.__meta[metakey] = [metadata]
        else:
            self.__meta[metakey].append(metadata)

    def getRawMeta(self):
        """\
        Get the raw meta data.

        Returns a hash to arrays.
        """
        return self.__meta

    def getMetaArray(self, metakey):
        """\
        Get the meta information as array.

        metakey - The keyword

        Returns an array which may be empty on undefined keys
        """
        if self.__meta.has_key(metakey):
            return self.__meta[metakey]
        else:
            return []
        
    def getMetaCommaSeparated(self, metakey):
        """\
        Get the meta information comma separated.
       
        metakey - The keyword
        """
        return ', '.join(self.getMetaArray(metakey))
            
    def getTitle(self):
        """\
        Get the title for the Blog.
        """
        if not self.__meta.has_key('title'):
            self.logError('No title in meta data')
            raise MissingTitle('no title for to %s' % (self.__blogname,))
        a = self.__meta['title']
        if len(a) > 1:
            self.logError('Title specified twice in meta data')
            raise DoubleTitle('title not unique to %s' % (self.__blogname,))
        return a[0].strip()

    def getType(self):
        """\
        Get the type of this meta part.

        It should be page or post, perhaps something more in future.

        If no type is given, it will be a post by default. This is the
        most frequently used type.
        """
        if not self.__meta.has_key('type'):
            self.logInfo('No type given, defaulting to post')
            t = 'post'
        else:
            a = self.__meta['type']
            if len(a) > 1:
                self.logError('Type specified twice')
                raise DoubleType('type not unique to %s' % (self.__blogname,))
            # Make lowercase for error tolerance -- users are strange
            t = a[0].lower() 
        if t not in ('post', 'page'):
            self.logError('Unknown type %s', t)
            raise IllegalType('unsupported type %s' % (t,))
        return t

    def getTypeInstance(self):
        """\
        Get the matching instance for the chose type of the meta.
        """
        # pychecker says:
        # Functions return types are inconsistent
        # This is intended.
        t = self.getType()
        if t == 'post':
            return wptypes.Post()
        elif t == 'page':
            return wptypes.Page()
        else:
            raise NotImplementedError('unsupported type %s' % (t, ))

    def getBlogConfig(self):
        """Get the blog config for this meta part"""
        return self.__bc

    def getBlogName(self):
        """Get the blog name for this meta part"""
        return self.__blogname


class InputParser(wpshbase.Base):
    """\
    The input parser.

    This is the workhorse for textfile posting. It is ugly code, but it
    must handle human input which may be irregular. The "uglyness" of the
    code reflects the computers degree of stupidity.
    """

    # A comment line
    # Changed: '#' must be first character in a comment line. It is a
    # possible char in quoted code and such lines can be introduced with
    # a single space now.
    com_re = re.compile(r'^\#[^#]')
    # Sometimes the masking with a space may be unwanted, so '##' at the
    # first two columns will be replaced by a '#'.
    com2_re = re.compile(r'^\#\#')
    # A wpcmd meta line
    met_re = re.compile(r'\s*\!\!([a-z]*)\s*\:\s*(.*)')
    # An encoding specification
    enc_re = re.compile(r'\-\*\-\s*coding\:\s*([0-9a-z\-]+)\s*\-\*\-', re.I)
    # A starting pre
    pre_start_re = re.compile(r'\<pre[^\>]*\>', re.I)
    # An ending pre
    pre_end_re = re.compile(r'\<\/pre[^\>]*\>', re.I)

    def __init__(self):
        """Constructor."""
        self.logDebug('Constructor')
        self.__text = ''
        self.__parag = []
        self.__mparts = {}
        self.__aktmp = None

    def __addMeta(self, matchobj):
        """Add a matched meta."""
        k = matchobj.group(1).lower()
        ln = matchobj.group(2)
        # Define a new blog context
        if k == 'blog':
            if self.__mparts.has_key(ln):
                self.__aktmp = self.__mparts[ln]
            else:
                self.__aktmp = InputMetaPart(ln)
                self.__mparts[ln] = self.__aktmp
        # Or add lines to the blog context
        else:
            if not self.__aktmp:
                raise NoBlogContext('no blog specified for %s', (ln,))
            self.__aktmp.addMeta(k, ln)

    # This method does a lot of work and is messy. I hope the comments
    # helps understanding it...
    def readFileObject(self, fileobj):
        """\
        Read a file given by a file object.

        fileobj - The file object
        """
        enc = wpshbase.LOCALENC.lower()
        self.logDebug('Reading text from file object')
        la = fileobj.readlines()
        # Look for encoding spec before processing
        for ln in la:
            mo = self.enc_re.search(ln)
            if mo:
                enc = mo.group(1)
                self.logInfo('Specified encoding %s found', enc)
                break
        # Decode a given encoding 
        if enc != wpshbase.LOCALENC.lower():
            self.logDebug('Recoding text')
            la = [ln.decode(enc) for ln in la]
        # Remove comment lines and strip lines, don't need these
        # trailing newlines
        la = [ln for ln in la if not self.com_re.match(ln)]
        # Replace leading '##' by '#' to make source publishing
        # more possible...
        la = [self.com2_re.sub('#', ln) for ln in la]
        # Process the lines and separate text from meta
        # Accumulate the lines in txtl
        self.logDebug('Separating text from meta specifications')
        txtl = []
        for ln in la:
            mo = self.met_re.match(ln)
            if mo:
                self.__addMeta(mo)
            else:
                txtl.append(ln)
        # Now we have textlines and meta parts separated, and the
        # meta parts are fine, so we should be ready to process 
        # the text. All stuff in a paragraph should become a long
        # line, this is the way wordpress can handle it automatically.
        parag = []
        work = []
        premode = False
        prebuf = ''
        # The loop is more complicated now, to proper handle that
        # <pre>formatted stuff. Sometimes I wanna blog sources.
        # If there are more internal parser modes in a future version,
        # this has to be done better, but for now the quick and dirty
        # approach is good.
        for ln in txtl:
            if premode:
                # A line of paranoia
                if self.pre_start_re.search(ln):
                    self.logError('Open pre tag in pre section')
                    raise MultiplePreTag('pre-tag encapsulated in pre-tag')
                # In pre-mode, lines are totally unfiltered
                prebuf += ln
                if self.pre_end_re.search(ln):
                    parag.append(prebuf)
                    premode = False
            else:
                if self.pre_start_re.search(ln):
                    if len(work) > 0:
                        parag.append(' '.join(work))
                        work = []
                    prebuf = ln
                    premode = True
                else:
                    ln = ln.strip()
                    if len(ln) == 0:
                        # This is an empty line, 
                        # so we put the work as paragraph
                        # if there is any.
                        if len(work) > 0:
                            parag.append(' '.join(work))
                            work = []
                    else:
                        # This is a non-empty line, 
                        # append it to the worker array.
                        work.append(ln)
        # There may be more work to do after the loop, don't want to lose
        # the last paragraph
        if len(work) > 0:
            parag.append(' '.join(work))
        self.__parag = parag
        # A last paranoia test
        if premode:
            self.logError('Open pre tag is not closed')
            raise UnclosedPreTag('pre-tag is not closed')
        # Now we have an array of paragraphs we can simply join together.
        # WordPress will do that little magic to format them back as
        # paragraphs.
        self.__text = '\n\n'.join(parag)

    def readFile(self, filename):
        """\
        Read a file given by name.

        filename - The filename
        """
        self.logInfo('Reading from file %s', filename)
        f = open(filename, 'rU')
        self.readFileObject(f)
        f.close()

    def getMetaParts(self):
        """Get the meta parts"""
        if len(self.__mparts) == 0:
            self.logError('No meta information found')
            raise MissingMeta('No meta information in the input')
        return self.__mparts.values()

    def getText(self):
        """Get the text"""
        return self.__text

    def getParagraphs(self):
        """Get the paragraphs of the text."""
        return self.__parag



########################################################################
#
# $Log: blog.py,v $
# Revision 1.16  2012/04/27 02:49:29  elias
# Changed style of raise stmts
#
# Revision 1.15  2012/04/24 21:43:39  elias
# Removed plugin stuff.
#
# Revision 1.14  2010/09/29 20:13:56  elias
# Logging.
#
# Revision 1.13  2010/09/26 19:09:22  elias
# Intermediate state after handling umlaut problems.
#
# Revision 1.12  2010/09/24 22:46:08  elias
# Cleanup for 0.4beta
#
# Revision 1.11  2010/09/24 20:15:09  elias
# Retrying after failed uploads or publishing attempts.
#
# Revision 1.10  2010/09/24 16:09:01  elias
# Removed plugin slot for password "encryption".
# Allowed '#' at the start of a line by typing '##'
#
# Revision 1.9  2010/02/10 18:58:52  elias
# Cleanup for 0.3alpha.
#

# Revision 1.8  2009/09/29 21:42:29  elias
# Fixed line-ending parsing bugs for DOS and Macintosh...
#
# Revision 1.7  2009/09/22 22:55:44  elias
# Fixed bug with <pre>-formatted text.
#
# Revision 1.6  2009/09/20 20:05:58  elias
# Source cleaned up for first alpha release.
#
# Revision 1.5  2009/09/20 04:08:33  elias
# Changed identification of the correct blog in BlogConfig.validate().
# The blogid in WordPress' RSD is wrong! Use with a WPMU, and disaster.
# I check for the URL in the list of wpshbase.getUserBlogs() now, which isn't
# backward compatible.
#
# Revision 1.4  2009/09/20 02:22:05  elias
# publishContent publishes Posts with uploads.
#
# Revision 1.3  2009/09/20 00:38:08  elias
# Milestone: can post but not upload.
#
# Revision 1.2  2009/09/18 22:07:05  elias
# Intermediate state.
# Local blog administration completed.
#
# Revision 1.1  2009/09/16 20:31:45  elias
# Initial revision
#
########################################################################
#
# CODE IS PROSA, NOT POETRY!
#
########################################################################
