#!/usr/bin/env python
# -*- coding: utf-8 -*-
########################################################################
#
# wpsh/commands.py
#
# wpcmd - A WordPress command line client
# Copyright (c) 2009-2012 Elias Schwerdtfeger, http://www.tamagothi.de/ 
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
#
########################################################################

"""\
Commands

For every command a little function is given, which implements it.

In case of an exception, the function pointed by exchandler is called,
which may be overwritten. 

The verbose messages are given by the function pointed by vmsg, which
may be overwritten. Your own implementation should respect the verbose
variable and be silent if no verbose output is requested. Do not use
any kind of output which blocks the user, no dialogs or similar stuff.

None of the functions is this module have any return value. They just
implement commands, sometimes a little interactively.

"""

from __future__ import print_function



ID = '$Id: commands.py,v 1.12 2012/04/27 20:25:31 elias Exp $'



import sys
import os
import os.path
import locale
import re
import codecs
import atexit
import webbrowser

import blog
import wpshell
import wpshbase
import wptypes



verbose = False
verbose_file = sys.stdout
verbose_mustclose = False
theConfig = blog.theConfig



class BlognameAlreadyExists(wpshbase.Error):
    """Avoid double blognames"""
    pass



class MalformedIdentifier(wpshbase.Error):
    """Some text is malformed"""
    pass



class CategoryError(wpshbase.Error):
    """A general category error"""
    pass



class CategoryDoesNotExist(CategoryError):
    """A cat does not exist"""
    pass



class CategoryAlreadyExists(CategoryError):
    """A category already exists"""
    pass



__excexit = False



def __exception(msg, *args):
    """\
    Handle an exception.
    """
    (typ, val, tb) = sys.exc_info()
    if wpshbase.DEBUG:
        # This isn't the wanted kind of output for a normal user!
        import traceback
        traceback.print_exception(typ, val, tb)
    doc = typ.__doc__
    sys.stderr.write('%s - %s\n' % (doc, val))
    rmsg = msg % args
    sys.stderr.write('%s\n' % (rmsg,))
    if __excexit:
        exit('Failure')



def __vmsg(fmt, *args):
    """\
    Print a verbose message.
    """
    if verbose:
        txt = fmt % args
        verbose_file.write('INFO: %s\n' % (txt,))



def __attmsg(fmt, *args):
    """\
    Print an important message.
    """
    txt = fmt % args
    verbose_file.write('ATTN: %s\n' % (txt,))



def __done():
    """\
    Say that everything is okay in verbose mode.
    """
    vmsg('Done.')



def __uinput(prompt, *args):
    """\
    A local input function.
    """
    p = prompt % args
    d = raw_input(p)
    return d



exchandler = __exception
vmsg       = __vmsg
done       = __done
attmsg     = __attmsg
uinput     = __uinput
pinput     = wpshbase.pinput



def __closeVerboseFile():
    global verbose_file
    if verbose_file != sys.stdout:
        verbose_file.close()
        verbose_file = sys.stdout



def setVerboseFile(filename):
    global verbose, verbose_file, verbose_mustclose
    try:
        __closeVerboseFile()
        verbose = True
        f = codecs.open(filename, 'w', wpshbase.LOCALENC)
        if not verbose_mustclose:
            atexit.register(__closeVerboseFile)
            verbose_mustclose = True
        verbose_file = f
    except IOError:
        exchandler('Unable to open message output file %s', filename)



def enforceReturnValueOnException():
    global __excexit
    __excexit = True



def listBlogs():
    try:
        tf = wpshbase.TupleFormatter()
        vmsg('List of configured blogs')
        blst = theConfig.getBlogList()
        if len(blst) == 0:
            print('There are no blogs configured yet')
        else:
            for i in blst:
                bc = theConfig.getBlogConfig(i)
                tf.add(bc.name, bc.comment)
            tf.printList()
        done()
    except Exception:
        exchandler('Failed to list blogs.')



def addBlog(name=None, usr=None, pwd=None, url=None, com=None):
    vmsg('Add and configure a blog')
    if not name:
        vmsg('Make the name short and unique, you will type it often')
        name = uinput('Name of the blog: ').strip()
    if theConfig.getBlogList().count(name) != 0:
        raise BlognameAlreadyExists('%s IS configured' % (name,))
    if name.count(' ') > 0:
        raise MalformedIdentifier('spaces are not allowed for blog names')
    if not usr:
        vmsg('The login name of your blog account')
        usr = uinput('Username: ').strip()
    if not pwd:
        vmsg('The password of your blog account')
        pwd = pinput('Password: ').strip()
    if not url:
        vmsg('The URL of your blog')
        url = uinput('Blog URL: ').strip()
    if not com:
        vmsg('A comment helping you to identify this blog')
        com = uinput('Comment: ').strip()
    try:
        vmsg('Adding %s...', name)
        vmsg('Autoconfiguring %s...', name)
        bc = blog.configureBlog(name, url, usr, pwd, com)
        vmsg('Saving %s to configuration...', name)
        done()
        theConfig.saveBlogConfig(bc)
    except Exception:
        exchandler('Failed to add blog configuration "%s".', name)



def removeBlog(name):
    try:
        if not name:
            name = uinput('Name of the blog to remove: ').strip()
        vmsg('Removing %s from the configuration...', name)
        theConfig.deleteBlogConfig(name)
        done()
    except Exception:
        exchandler('Failed to remove blog configuration "%s".', name)



def visitBlog(name):
    try:
        if not name:
            name = uinput('Name of the blog to visit: ').strip()
        vmsg('Visiting %s with the web browser.', name)
        bc = theConfig.getBlogConfig(name)
        bc.visit()
        done()
    except Exception:
        exchandler('Failed to visit configured blog "%s".', name)



def blogInformation(name):
    try:
        if not name:
            name = uinput('Name of the blog for details: ').strip()
        vmsg('Blog information for %s', name)
        bc = theConfig.getBlogConfig(name)
        tf = wpshbase.TupleFormatter()
        tf.add('Name', bc.name)
        tf.add('URL', bc.url)
        tf.add('Blog-ID', bc.id)
        tf.add('XMLRPC-URL', bc.xmlurl)
        tf.add('Categories', ', '.join(bc.cats.keys()))
        tf.add('Admin-Flag', bc.admin)
        tf.add('Comment', bc.comment)
        tf.printList()
        done()
    except Exception:
        exchandler('Failed to get blog information for "%s".', name)



def revalidateBlog(name):
    try:
        if not name:
            name = uinput('Name of the blog to revalidate: ').strip()
        vmsg('Revalidating %s...', name)
        bc = theConfig.getBlogConfig(name)
        if not bc.revalidate():
            raise RuntimeError('Failed to validate')
        vmsg('Saving the revalidated configuration...')
        theConfig.saveBlogConfig(bc)
        done()
    except Exception:
        exchandler('Failed to revalidate blog configuration "%s".', name)



def changePassword(name):
    try:
        if not name:
            name = uinput('Name of the blog for changing password: ').strip()
        bc = theConfig.getBlogConfig(name)
        pwneu = pinput('New password for %s: ', name).strip()
        vmsg('Changing password...')
        bc.pwd = pwneu
        vmsg('Trying to revalidate for password check...')
        if not bc.revalidate():
            raise RuntimeError('No login with the new password')
        theConfig.saveBlogConfig(bc)
        done()
    except Exception:
        exchandler('Failed to change password')



def blogReadCategories(name):
    try:
        if not name:
            name = uinput('Name of the blog to read categories from: ').strip()
        vmsg('Reading categories for %s', name)
        bc = theConfig.getBlogConfig(name)
        bc.readCategories()
        theConfig.saveBlogConfig(bc)
        done()
    except Exception:
        exchandler('Failed to read categories for "%s".', name)



def blogRename(oldname, newname):
    try:
        vmsg('Renaming configuration %s to %s', oldname, newname)
        if theConfig.getBlogList().count(newname) > 0:
            raise BlognameAlreadyExists('%s already exists' % (newname,))
        theConfig.renameBlogConfig(oldname, newname)
        done()
    except Exception:
        exchandler('Failed to rename configuration.')



def blogComment(name, comment):
    try:
        vmsg('Setting comment for %s', name)
        bc = theConfig.getBlogConfig(name)
        bc.comment = comment
        vmsg('Saving modified blog configuration')
        theConfig.saveBlogConfig(bc)
        done()
    except Exception:
        exchandler('Failed to modify blog comment.')



def interactiveMode():
    try:
        vmsg('Starting interactive mode')
        sh = wpshell.WPShell()
        sh.cmdloop()
    except Exception:
        exchandler('Unhandled exception. This should never happen.');



def categoryTree(name):
    try:
        if not name:
            name = uinput('Name of the blog: ').strip()
        vmsg('Getting category tree')
        bc = theConfig.getBlogConfig(name)
        ct = bc.getCategoryTree()
        ct.printTree()
        done()
    except Exception:
        exchandler('Failed to get category tree for "%s".', name)



def addCategory(bc, parentname = None):
    try:
        vmsg('Adding category to blog %s', bc.name)
        c = wptypes.Category()
        c.categoryName = uinput('Category Name: ').strip()
        if bc.cats.has_key(c.categoryName):
            raise CategoryAlreadyExists('%s already exists' % (c.categoryName,))
        c.description = uinput('Description: ').strip()
        if parentname:
            vmsg('Retrieving ID for parent %s', parentname)
            if not bc.cats.has_key(parentname):
                raise CategoryDoesNotExist('not category %s' % (parentname,))
            pc = bc.cats[parentname]
            c.parentId = pc.categoryId
        vmsg('Creating new category')
        bc.newCategory(c)
        theConfig.saveBlogConfig(bc)
        attmsg("The category isn't configured. Do a 'config categories' soon")
        done()
    except Exception:
        exchandler('Failed to add category.')



def removeCategory(bc, name):
    try:
        vmsg('Removing category %s', name)
        if not bc.cats.has_key(name):
            raise CategoryDoesNotExist('no category %s' % (name,))
        c = bc.cats[name]
        bc.removeCategory(c)
        theConfig.saveBlogConfig(bc)
        done()
    except Exception:
        exchandler('Failed to remove category.')



def visitCategory(bc, name):
    try:
        vmsg('Visiting category %s', name);
        if not bc.cats.has_key(name):
            raise CategoryDoesNotExist('no category %s' % (name,))
        cat = bc.cats[name]
        cat.visit()
        done()
    except Exception:
        exchandler('Failed to visit category.')



def blogPostTemplate(pattern):
    try:
        vmsg('Creating post templates for pattern "%s"', pattern)
        p = re.escape(pattern).replace(r'\*', '.*')
        flag = False
        for blogname in theConfig.getBlogList():
            if re.search(p, blogname):
                fname = blogname + '-Post.txt'
                vmsg('Creating %s for %s', fname, blogname)
                bc = theConfig.getBlogConfig(blogname)
                f = codecs.open(fname, 'wt', wpshbase.LOCALENC)
                f.write(bc.createPostTemplate())
                f.close()
                flag = True
        if not flag:
            attmsg('None of the configured blogs matches "%s"', pattern)
        done()
    except Exception:
        exchandler('Failed to create post templates.')



# For future functionality
# There are no pages in wpcmd at the moment...

def blogPageTemplate(pattern):
    try:
        vmsg('Creating page templates for pattern "%s"', pattern)
        p = re.escape(pattern).replace(r'\*', '.*')
        flag = False
        for blogname in theConfig.getBlogList():
            if re.search(p, blogname):
                fname = blogname + '-Page.txt'
                vmsg('Creating %s for %s', fname, blogname)
                bc = theConfig.getBlogConfig(blogname)
                f = codecs.open(fname, 'wt', wpshbase.LOCALENC)
                f.write(bc.createPageTemplate())
                f.close()
                flag = True
        if not flag:
            attmsg('None of the configured blogs matches "%s"', pattern)
        done()
    except Exception:
        exchandler('Failed to create page templates')



def postFile(filename):
    try:
        inputParser = blog.InputParser()
        if filename == '-':
            vmsg('Posting from stdin')
            inputParser.readFileObject(sys.stdin)
            filename = '<stdin>'
        else:
            vmsg('Posting %s', filename)
            inputParser.readFile(filename)
        for metaPart in inputParser.getMetaParts():
            bc = metaPart.getBlogConfig()
            vmsg('Posting to %s', bc.name)
            instance = metaPart.getTypeInstance()
            instance.feed(metaPart, inputParser)
            post_id = bc.publishContent(instance, vmsg)
            vmsg('The WordPress-ID of the posting is %d', post_id)
        done()
    except Exception:
        exchandler('Failed to publish file')



def cryptConfigString(s):
    print(blog.theConfig.encryptLogin((s,))[0])



def visitTutorial():
    try:
        # For more languages, this must be coded better.
        loc, enc = locale.getdefaultlocale()
        if loc.startswith('de'):
            tutfile = 'Liesmich.html'
        else:
            tutfile = 'Readme.html'
        # Better check environment before using it ;-)
        if os.environ.has_key('WPCMD_DOCDIR'):
            tutdir = os.environ['WPCMD_DOCDIR']
            tuturl = 'file:///%s/html/%s' % (tutdir, tutfile)
        else:
            tutdir = os.path.join(os.getcwd(), 'doc')
            tuturl = 'file:///%s/%s' % (tutdir, tutfile)
        webbrowser.open(tuturl)
    except Exception:
        exchandler('Failed to open tutorial file in the browser')


    
########################################################################
#
# $Log: commands.py,v $
# Revision 1.12  2012/04/27 20:25:31  elias
# Better except-spec
# Changed visitTutorial() to make reading w/o installation possible
#
# Revision 1.11  2012/04/27 02:49:41  elias
# Changed style of raise stmts
#
# Revision 1.10  2011/02/23 09:08:45  elias
# print function
#
# Revision 1.9  2010/09/26 19:08:17  elias
# Added the visitTutorial command.
#
# Revision 1.8  2010/09/26 16:38:19  elias
# Output file for verbose messages.
# (stdout with unicode chars raises errors when redirected)
#
# Revision 1.7  2010/09/24 22:49:55  elias
# Cleanup for 0.4beta
#
# Revision 1.6  2010/09/24 18:17:43  elias
# Cmd for changing password
# Invisible password input when possible
#
# Revision 1.5  2010/02/10 19:00:06  elias
# Added crypt function for helping experts in writing configuration
#
# Revision 1.4  2009/09/23 15:52:38  elias
# More user-centred error messages.
#
# Revision 1.3  2009/09/20 02:31:37  elias
# Milestone: Can post with uploads
#
# Revision 1.2  2009/09/18 22:08:10  elias
# Intermediate state.
# Complete local blog administration.
#
# Revision 1.1  2009/09/17 15:33:25  elias
# Initial revision
#
########################################################################
#
# CODE IS PROSA, NOT POETRY!
#
########################################################################
