#!/usr/bin/env python
# -*- coding: utf-8 -*-
########################################################################
#
# wpsh/wpshbase.py
#
# wpcmd - A WordPress command line client
# Copyright (c) 2009-2012 Elias Schwerdtfeger, http://www.tamagothi.de/ 
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
#
########################################################################

"""\
wpshbase.py

Base classes and utilities.

Part of this module are some hacks around future python compatibility
issues.

CfgParser  is the configuration parser class
XmlRpc     is the XMLRPC server proxy class
HtmlParser is the HTML parser class
CfgPath    is the path of the configuration file

MODULE VARIABLES:

DEBUG      -  A debugging flag
ID         -  Version identifier
VERSION    -  Version string
LICENSE    -  A license announcement
WARRANTY   -  A (no-)warranty announcement
DONATE     -  A short text how to support me
LOCALENC   -  The encoding used locally
"""

from __future__ import print_function

import locale
import logging
import sys
import os.path
import codecs
import re



# Set Debug to False in stable release versions!
DEBUG = False
ID = '$Id: wpshbase.py,v 1.13 2012/04/27 02:50:03 elias Exp elias $'
VERSION = '0.05'
LOCALENC = locale.getpreferredencoding()

LICENSE = """
wpcmd -- The command line driven WordPress shell.
Copyright (c) 2009-2012 Elias Schwerdtfeger, http://www.tamagothi.de/

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the 

    Free Software Foundation, Inc., 
    59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA."""

WARRANTY = """\
BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WAR‐
RANTY FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW.
EXCEPT WHEN OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS
AND/OR OTHER PARTIES PROVIDE THE PROGRAM ‘‘AS IS’’ WITHOUT WARRANTY
OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIM‐
ITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE.  THE ENTIRE RISK AS TO THE QUALITY AND PER‐
FORMANCE OF THE PROGRAM IS WITH YOU.  SHOULD THE PROGRAM PROVE
DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR
OR CORRECTION."""

DONATE = """\
If you like this nerdy little program, which empowers you to do
many WordPress tasks on a command line and even to do scripted
blogging in an easy way, please support me and the further deve-
lopment of this program.

I am an homeless artist and live by begging. I just give away all
the things I do and create, and I hope that something will return
to me. I may sound naive, but I can't live another way.

You can make your little donation via PayPal on my german homepage.
Just visit http://www.tamagothi.de/ and look for the sidebar ele-
ment titled "Spende", which is the german word for "donation."

And if you like it, take all the other stuff available there... ;-)"""



CfgParser  = None
HtmlParser = None
XmlRpc     = None 
XmlBinary  = None
XmlFault   = None
CfgPath    = None



if sys.hexversion >= 0x30000000:
    import configparser
    CfgParser = configparser.ConfigParser
else:
    import ConfigParser
    CfgParser = ConfigParser.ConfigParser

if sys.hexversion >= 0x30000000:
    import xmlrpc.client
    XmlRpc    = xmlrpc.client.ServerProxy
    XmlBinary = xmlrpc.client.Binary
    XmlFault  = xmlrpc.client.Fault
else:
    import xmlrpclib
    XmlRpc    = xmlrpclib.ServerProxy
    XmlBinary = xmlrpclib.Binary
    XmlFault  = xmlrpclib.Fault

if sys.hexversion >= 0x30000000:
    import html.parser
    HtmlParser = html.parser.HTMLParser
else:
    import HTMLParser
    HtmlParser = HTMLParser.HTMLParser

import warnings
import getpass
import logging



def pinput(prompt, *args):
    """\
    Helper function for invisible password input.

    prompt - The prompt string
    args - Arguments for formatting the string

    Returns the invisible typed password
    """
    p = prompt % args
    # Don't want a warning message...
    with warnings.catch_warnings():
        warnings.simplefilter('ignore')
        i = getpass.getpass(p)
    return i



# TODO: Better path and filename for other platforms than Unices.
# (How does ~ expand on Windows? What's about the dotted file?)

CfgPath = os.path.expanduser(os.path.join('~', '.wpcmdrc'))
LogPath = os.path.expanduser(os.path.join('~', 'wpcmd.log'))

LoggingConfigured = False
LOGMSGFORMAT      = '%(asctime)s:%(levelname)s:%(name)s:%(message)s'
LOGDATEFORMAT     = '%Y-%m-%d %H.%M.%S'

CLSSTRRE = re.compile(r'[^\']+\'([^\']+)\'')



# Please note that the doc string is displayed to the user in error messages.
# That's the reason for my so many exception classes, it makes printing
# errors much easier to me.
#
# If you hack in this code and there is a need for additional exceptions,
# give good doc-strings, roughly addressed to simple users who don't 
# understand nothing of all this hacking stuff and just wanna blog something
# on the commandline (or from an application).

class Error(RuntimeError):
    "Error"
    pass



class InternalError(Error):
    "Internal Error"
    pass



class DoubleKey(InternalError):
    "A unique name is specified twice"
    pass



def changeLogPath(newpath):
    """\
    Change the path for the logfile.

    This must be done before the first log file entry is written.
    """
    global LogPath
    if LoggingConfigured:
        raise InternalError('Cannot change logfile path')
    LogPath = newpath



class Base(object):
    """\
    A generic base class.

    There is no need for __init__ in this class. It encapsulates just
    some useful functions for derived classes.

    LOGGING:
    
    A simple interface for logging is implemented here too. It is 
    still very basic, but may be extended in the future. Logging may
    be an essential feature for automated processes...

    CLASSNAME:

    Getting the full qualified name of the class. This is used in logging.
    """

    @classmethod
    def getClassName(cls):
        """\
        Get the full qualified name of the class.

        Returns the name of the class
        """
        if not hasattr(cls, '__class_name'):
            cname = str(cls)
            mo = CLSSTRRE.search(cname)
            if mo:
                cls.__class_name = mo.group(1)
            else:
                # It should never never happen.
                # This is just a line of paranoia code to handle
                # future changes in the string conversion of
                # classes. The format isn't documented, and so
                # the regex may fail.
                cls.__class_name = cname
        return cls.__class_name

    @classmethod
    def getLogger(cls):
        """\
        Get the logger instance for this class.

        Returns logger instance
        """
        global LoggingConfigured
        # It is created when requested for the first time. This allows
        # changing the settings from parsed commandline without the need
        # to reconfigure the logging options.
        if not LoggingConfigured:
            if DEBUG:
                log_level = logging.DEBUG
            else:
                log_level = logging.INFO
            logging.basicConfig(filename=LogPath,
                                level=log_level,
                                format=LOGMSGFORMAT,
                                datefmt=LOGDATEFORMAT)
            LoggingConfigured = True
        if not hasattr(cls, '__logger'):
            cls.__logger = logging.getLogger(cls.getClassName())
        return cls.__logger

    @classmethod
    def logDebug(cls, fmt, *param):
        """\
        Log a debug message.
        """
        l = cls.getLogger()
        l.debug(fmt % param)

    @classmethod
    def logInfo(cls, fmt, *param):
        """\
        Log an informational message.
        """
        l = cls.getLogger()
        l.info(fmt % param)

    @classmethod
    def logWarning(cls, fmt, *param):
        """\
        Log a warning message.
        """
        l = cls.getLogger()
        l.warning(fmt % param)

    @classmethod
    def logError(cls, fmt, *param):
        """\
        Log an error message.
        """
        l = cls.getLogger()
        l.error(fmt % param)

    @classmethod
    def logException(cls, fmt, *param):
        """\
        Log an exception.
        """
        l = cls.getLogger()
        l.exception(fmt % param)



class TupleFormatter(Base):
    """\
    A helper class for printing Tuples of information.

    Instances of this class accumulates the tuples with the add() method,
    the sorted list is printed with the printList method. 
    """

    def __init__(self):
        """\
        Constructor.
        """
        self.__a = {}

    def add(self, key, value):
        """\
        Add a tupel to the printer instance.

        key - The key
        value - The associated text or other stuff
        """
        key = unicode(key, errors='ignore')
        if self.__a.has_key(key):
            raise DoubleKey('Double key value "%s"' % (key,))
        self.__a[key] = value
        
    def printList(self, indent=0):
        """\
        Print the list.

        indent - Number of spaces for indentation (optional)
        """
        lst = []
        keys = self.__a.keys()
        maxlen = max([len(i) for i in keys])
        firstfmt = u"%%-%ds" % (maxlen,)
        keys.sort(lambda x, y: cmp(x.lower(), y.lower()))
        for i in keys:
            if indent > 0:
                print(' ' * indent, firstfmt % (i,), '-', self.__a[i])
            else:
                print(firstfmt % (i,), '-', self.__a[i])



########################################################################
#
# $Log: wpshbase.py,v $
# Revision 1.13  2012/04/27 02:50:03  elias
# Changed style of raise stmts
#
# Revision 1.12  2012/04/24 23:00:32  elias
# Changed the whole thing to new style classes.
# Not necessary now.
#
# Revision 1.11  2012/04/24 21:46:32  elias
# Removed obsolete plugins stuff
#
# Revision 1.10  2011/02/23 09:09:53  elias
# Print function.
#
# Revision 1.9  2010/09/29 20:12:47  elias
# Added Logging to base class.
# Added interface to set the log file initially.
#
# Revision 1.8  2010/09/26 19:08:46  elias
# Intermediate state after handling umlaut problems.
#
# Revision 1.7  2010/09/24 22:52:50  elias
# Cleanup for 0.4 beta
#
# Revision 1.6  2010/09/24 20:35:51  elias
# Password input function added
#
# Revision 1.5  2010/02/10 18:39:32  elias
# Incremented version number... ;-)
#
# Revision 1.4  2009/09/23 15:55:46  elias
# Source cleanup for Alpha2
#
# Revision 1.3  2009/09/20 20:09:07  elias
# Source cleaned up for alpha release.
# DEBUG set to False.
#
# Revision 1.2  2009/09/20 02:22:34  elias
# Milestone: Post with binary Uploads
#
# Revision 1.1  2009/09/17 11:45:07  elias
# Initial revision
#
########################################################################
#
# CODE IS PROSA, NOT POETRY!
#
########################################################################

